/*
 * SOFTSPI.c
 *
 * Created: 10.01.2013 09:16:25
 *  Author: Behlau
 */ 

#include "SOFTSPI.h"

#define F_CPU	16000000UL	//16MHz
#include <util/delay.h>
#include <avr/io.h>
#include <inttypes.h>



// Initalisierung 
void SOFTSPI_Init(void)
{
  // MOSI und CLK auf Ausgang setzen
  SOFTSPI_MOSI_DDR |= SOFTSPI_MOSI_BIT;
  SOFTSPI_SCLK_DDR |= SOFTSPI_SCLK_BIT;

  // MOSI und CLK auf HIGH setzen
  SOFTSPI_MOSI_PORT |= SOFTSPI_MOSI_BIT;
  SOFTSPI_SCLK_PORT |= SOFTSPI_SCLK_BIT;

  // MISO auf Eingang setzen
  SOFTSPI_MISO_DDR &= ~SOFTSPI_MISO_BIT;
}


void SOFTSPI_Clock_Mode(char spi_mode)
{
	if( ( spi_mode & 0x02 ) == 0x00 )		// Clock = IDLE LOW!
		SOFTSPI_SCLK_PORT &= ~SOFTSPI_SCLK_BIT;
	else									// Clock = IDLE HIGH!
		SOFTSPI_SCLK_PORT |= SOFTSPI_SCLK_BIT;			
}

unsigned char SOFTSPI_WriteRead_Synchron(unsigned char dataout, char spi_mode)
{
	SOFTSPI_Clock_Mode(spi_mode);
	_delay_us(1);
	
	uint8_t datain=0;
	//das Byte wird Bitweise nacheinander gesendet, MSB zuerst
	for (uint8_t a=8; a>0; a--)
	{                
		datain<<=1;											//Schieben um das Richtige Bit zusetzen

		// Daten schreiben Versuch 1
		if( ( spi_mode & 0x01 ) == 0x00 )		// CPHA = 0 --> Daten rauschreiben vor 1ten Flanke!
		{
			if (dataout & 0x80)
				SOFTSPI_MOSI_PORT |= SOFTSPI_MOSI_BIT;			//Set Output High
			else
				SOFTSPI_MOSI_PORT &= ~SOFTSPI_MOSI_BIT;			//Set Output Low
		}

		_delay_us(1);

		// Erste Flanke!
		if( ( spi_mode & 0x02 ) != 0x00 )		// CPOL = 1
			SOFTSPI_SCLK_PORT &= ~SOFTSPI_SCLK_BIT;				// CPHA = 1 --> Erste Flanke = Fallende Flanke
		else
			SOFTSPI_SCLK_PORT |= SOFTSPI_SCLK_BIT;				// CPHA = 0 --> Erste Flanke = Steigende Flanke
			
		// Daten schreiben Versuch 2
		if( ( spi_mode & 0x01 ) != 0x00 )		// CPHA = 1
		{
			// Daten rauschreiben nach 1ten Flanke!
			if (dataout & 0x80)
				SOFTSPI_MOSI_PORT |= SOFTSPI_MOSI_BIT;			//Set Output High
			else
				SOFTSPI_MOSI_PORT &= ~SOFTSPI_MOSI_BIT;			//Set Output Low
		}
		
		_delay_us(1);
		
		// Daten lesen Versuch 1
		if( ( spi_mode & 0x01 ) == 0x00 )		// CPHA = 0
		{	
			// Daten lesen nach erster Flanke!
			if (SOFTSPI_MISO_PIN & SOFTSPI_MISO_BIT)			//Lesen des Pegels
				datain |= 1;
		}
			
		// Zweite Flanke!
		if( ( spi_mode & 0x02 ) != 0x00 )		// CPOL = 1
			SOFTSPI_SCLK_PORT |= SOFTSPI_SCLK_BIT;				// CPHA = 1 --> Zweite Flanke = Steigende Flanke
		else
			SOFTSPI_SCLK_PORT &= ~SOFTSPI_SCLK_BIT;				// CPHA = 0 --> Zweite Flanke = Fallende Flanke


		_delay_us(1);

		// Daten lesen Versuch 2
		if( ( spi_mode & 0x01 ) != 0x00 )		// CPHA = 1
		{	
			// Daten lesen nach erster Flanke!
			if (SOFTSPI_MISO_PIN & SOFTSPI_MISO_BIT)			//Lesen des Pegels
				datain |= 1;
		}

		dataout<<=1;										//Schiebe um nchstes Bit zusenden
	}

	return datain;
}
